"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.openApiToMockFile = exports.openApiToMockDefinition = void 0;
const RouteCreator_1 = require("./RouteCreator");
const AutoMockServer_1 = require("./AutoMockServer");
const swagger_parser_1 = __importDefault(require("@apidevtools/swagger-parser"));
const fs_1 = require("fs");
const MOCK_KEY = 'x-mock';
const MOCK_REF = 'x-mock-ref';
const META = Symbol('META');
function log(level, ...message) {
    if (level == 'error') {
        console.error(...message);
    }
    else if (level == 'warning') {
        console.warn(...message);
    }
    else {
        console.log(...message);
    }
}
function stringifyObject(data, level = 1) {
    const MARGIN = ' '.repeat(level);
    let json = '';
    if (typeof data === 'string' || data instanceof String) {
        json += `"${data}"`;
    }
    else if (Array.isArray(data)) {
        let res = data.map((e) => stringifyObject(e, level + 2));
        json += `\n${MARGIN} [${res.join(',')}]`;
    }
    else if (typeof data === 'object') {
        const entries = Object.entries(data);
        let symbolEntries = Object.getOwnPropertySymbols(data).map((s) => [
            s,
            data[s],
        ]);
        const metaIndex = symbolEntries.findIndex(([s]) => s === META);
        let meta = null;
        let filter = null;
        let bind = null;
        if (metaIndex >= 0) {
            meta = symbolEntries[metaIndex][1];
            if (meta.bind)
                bind = meta.bind;
            if (Array.isArray(meta.filter)) {
                filter = `getQueryParamFilter([${meta.filter
                    .map((s) => `"${s}"`)
                    .join(',')}])`;
            }
            else {
                filter = meta.filter;
            }
            symbolEntries.splice(metaIndex, 1);
        }
        let objStr = [...symbolEntries, ...entries].map(([k, v]) => {
            const isSymbol = typeof k == 'symbol';
            let key = isSymbol ? `[${k.description}]` : `"${k}"`;
            if (isSymbol && k.description == 'PROTOCOL' && Array.isArray(v))
                return `${key}:[${v.join(',')}]`;
            if (k == 'filter' && filter)
                return `filter:${filter}`;
            if (k == 'dynamicKeys' && bind)
                return `dynamicKeys:[${bind.join(',')}]`;
            return `${key}:${stringifyObject(v, level + 1)}`;
        });
        json += `{\n${MARGIN}${objStr.join(`,\n${MARGIN}`)}\n${MARGIN}}`;
    }
    else {
        json += `${String(data)}`;
    }
    return json;
}
function getMockFile(data) {
    return `
const { mock, PATH_SYMBOLS, METHOD } = require('auto-route-creator');
const { BIND, PROTOCOL, DYNAMIC, ROUTE, NAME, RESPONSE } = PATH_SYMBOLS;
const {${Object.values(RouteCreator_1.METHOD).join(',')}} = METHOD;

// used by filter (mainly query parameters) to find match
${String(searchNested)}

${String(getQueryParamFilter)}

// Route definitions
const ROUTES = ${stringifyObject(data)};


mock(ROUTES, {port: 3002, defaultListSize: 4});
`;
}
function searchNested(obj, key) {
    for (let k in obj) {
        if (k === key) {
            return obj[k];
        }
        if (typeof obj[k] === 'object') {
            const result = searchNested(obj[k], key);
            if (result) {
                return result;
            }
        }
    }
    return null;
}
function getQueryParamFilter(queryParams) {
    let filterList = queryParams.map((name) => {
        return (responseData, req) => {
            var _a;
            const searchValue = (_a = req === null || req === void 0 ? void 0 : req.query) === null || _a === void 0 ? void 0 : _a[name];
            if (searchValue) {
                if (Array.isArray(responseData))
                    return responseData.filter((obj) => searchNested(obj, name) == searchValue);
            }
            return responseData;
        };
    });
    return (reqData, req) => filterList.reduce((prev, cur) => cur(prev, req), reqData);
}
function parseParameters(methodParameters) {
    let value = (responseData) => responseData;
    if (!methodParameters)
        return { filter: value, queryParams: [] };
    let queryParams = methodParameters
        .filter((parameter) => parameter.in == 'query')
        .map(({ name }) => name);
    return { filter: getQueryParamFilter(queryParams), queryParams };
}
function parseResponse(data, mockRefs) {
    let type = data === null || data === void 0 ? void 0 : data.type;
    let mockRefId = data === null || data === void 0 ? void 0 : data[MOCK_REF];
    if (mockRefs[mockRefId])
        return mockRefs[mockRefId];
    if (data[MOCK_KEY])
        return data[MOCK_KEY];
    if (type == 'object') {
        return parseObject(data, mockRefs);
    }
    if (type == 'array') {
        return parseArray(data, mockRefs);
    }
    if (type == 'string') {
        return parseString(data);
    }
    if (type == 'integer') {
        return parseNumber(data);
    }
    if (type == 'boolean') {
        return parseBoolean();
    }
}
function parseObject(data, mockRefs) {
    let value = {};
    if (!data.properties)
        return value;
    Object.keys(data.properties).forEach((k) => {
        value[k] = parseResponse(data.properties[k], mockRefs);
    });
    return value;
}
function parseArray(data, mockRefs) {
    let value = [];
    let items = data.items;
    if (!items)
        return value;
    return [parseResponse(items, mockRefs)];
}
function parseBoolean() {
    return '{{datatype.boolean}}:' + AutoMockServer_1.TYPES.BOOL;
}
function parseString(property) {
    const format = property.format;
    let value = '';
    if (format) {
        if (format == 'date') {
            const DATE_RANGE = 1680000000000;
            value = () => new Date(Math.random() * DATE_RANGE).toISOString().split('T')[0];
        }
        if (format == 'date-time') {
            value = '{{date.past}}';
        }
        if (format == 'password') {
            value = '{{internet.password}}';
        }
        if (format == 'byte') {
            value = '{{random.alphaNumeric(50)}}';
        }
        if (format == 'binary') {
            value = '[100|101|110|1|10|111|0|11]';
        }
        if (format == 'email') {
            value = '{{internet.email}}';
        }
        if (format == 'uuid') {
            value = '{{datatype.uuid}}';
        }
        if (format == 'uri' || format == 'url') {
            value = '{{internet.url';
        }
        if (format == 'hostname') {
            value = '{{name.lastName}}';
        }
        if (format == 'ipv4') {
            value = '{{internet.ip}}';
        }
        if (format == 'ipv6') {
            value = '{{internet.ipv6}}';
        }
    }
    else if (Array.isArray(property.enum)) {
        value = `[${property.enum.join('|')}]`;
    }
    else if (property.example) {
        value = `[${property.example}]`;
    }
    else {
        //random word in case no info.
        value = '{{animal.dog}}';
    }
    return value + ':' + AutoMockServer_1.TYPES.STRING;
}
function parseNumber(property) {
    const format = property.format;
    let value = '';
    if (format) {
        if (format == 'float') {
            value = '{{datatype.float}}';
        }
        if (format == 'double') {
            value = '{{datatype.float(0.000000001)}}';
        }
        if (format == 'int32') {
            value = '{{datatype.number(1000000)}}';
        }
        if (format == 'int64') {
            value = '{{datatype.number(9223372036854775807)}}';
        }
    }
    else if (property.example) {
        const RANGE_MUL = 2;
        const DEFAULT_INT = 10;
        if (property.example.includes('.'))
            value = `{{datatype.number(${(Number(property.example) || DEFAULT_INT) * RANGE_MUL})}}`;
        else
            value = '{{datatype.float}}';
    }
    else {
        //random number.
        value = '`{{datatype.number}}';
    }
    return value + ':' + AutoMockServer_1.TYPES.NUMBER;
}
function openApiToMockDefinition(document, mockRefs = {}) {
    return __awaiter(this, void 0, void 0, function* () {
        const openApiDocument = yield swagger_parser_1.default.validate(document);
        let { paths } = openApiDocument;
        let routes = {};
        if (!paths) {
            log('error', 'Cannot convert open API spec, no paths found');
            return null;
        }
        const pathsList = Object.keys(paths);
        pathsList.forEach((path) => {
            const pathParts = path.split('/');
            let dynamicName = null;
            let curRoutes = routes;
            pathParts.forEach((part) => {
                if (part[0] == '{' && part[part.length - 1] == '}') {
                    dynamicName = true;
                    part = part.slice(1, part.length - 1);
                }
                console.log('PART:', part);
                if (part) {
                    if (!curRoutes[part])
                        curRoutes[part] = {};
                    curRoutes = curRoutes[part];
                }
            });
            //Add protocols (aka method)
            const methodList = Object.keys(paths[path]);
            let supportedMethods = [];
            methodList.forEach((method) => {
                const methodUpper = method.toUpperCase();
                //Move Get to be first definition (for clarity, as main use case)
                if (Object.keys(RouteCreator_1.METHOD).includes(methodUpper)) {
                    if (methodUpper == 'GET')
                        supportedMethods.unshift(method.toUpperCase());
                    else
                        supportedMethods.push(method.toUpperCase());
                }
                else
                    log('warning', `Warning, unsupported method ${method} (for path ${path})`);
            });
            //Add mock definition per method
            if (dynamicName)
                curRoutes[RouteCreator_1.SYMBOLS.DYNAMIC] = paths[path]['x-mock-dynamic'] || true;
            curRoutes[RouteCreator_1.SYMBOLS.PROTOCOL] = supportedMethods;
            curRoutes[RouteCreator_1.SYMBOLS.RESPONSE] = [];
            supportedMethods.map((method) => {
                var _a, _b, _c, _d, _e, _f;
                const methodData = paths[path][method.toLowerCase()];
                const dynamicBindListStr = methodData['x-mock-bind'];
                const operationId = methodData['operationId'];
                const methodParameters = methodData === null || methodData === void 0 ? void 0 : methodData['parameters'];
                const methodResponse = methodData === null || methodData === void 0 ? void 0 : methodData['responses'];
                const responseSchema = (_c = (_b = (_a = methodResponse === null || methodResponse === void 0 ? void 0 : methodResponse[200]) === null || _a === void 0 ? void 0 : _a['content']) === null || _b === void 0 ? void 0 : _b['application/json']) === null || _c === void 0 ? void 0 : _c['schema'];
                const responseSchemaDefault = (_f = (_e = (_d = methodResponse === null || methodResponse === void 0 ? void 0 : methodResponse['default']) === null || _d === void 0 ? void 0 : _d['content']) === null || _e === void 0 ? void 0 : _e['application/json']) === null || _f === void 0 ? void 0 : _f['schema'];
                if (methodData['operationId'] && mockRefs[operationId]) {
                    curRoutes[RouteCreator_1.SYMBOLS.RESPONSE].push(mockRefs[operationId]);
                }
                else if (responseSchema || responseSchemaDefault) {
                    let schema = responseSchema || responseSchemaDefault;
                    const res = parseResponse(schema, mockRefs);
                    const { filter, queryParams } = parseParameters(methodParameters);
                    let resDef = { template: res };
                    if (filter)
                        resDef['filter'] = filter;
                    if (!resDef[META])
                        resDef[META] = {};
                    resDef[META].filter =
                        (queryParams === null || queryParams === void 0 ? void 0 : queryParams.length) > 0 ? queryParams : (data) => data;
                    if (dynamicBindListStr) {
                        resDef[META].bind = [];
                        resDef['dynamicKeys'] = dynamicBindListStr.split(',').map((dBind) => {
                            const [dKey, dId] = dBind.split(':');
                            resDef[META].bind.push(`BIND("${dKey}","${dId}")`);
                            return (0, RouteCreator_1.BIND)(dKey, dId);
                        });
                    }
                    curRoutes[RouteCreator_1.SYMBOLS.RESPONSE].push(resDef);
                }
                else {
                    log('warning', `Warning, no response for ${method}:${path}`);
                    curRoutes[RouteCreator_1.SYMBOLS.RESPONSE].push({});
                }
            });
        });
        return routes;
    });
}
exports.openApiToMockDefinition = openApiToMockDefinition;
function openApiToMockFile(document, path) {
    return new Promise((res, rej) => {
        openApiToMockDefinition(document).then((routes) => {
            (0, fs_1.writeFile)(path, getMockFile(routes), function (err) {
                if (err) {
                    rej(err);
                }
                else {
                    res();
                }
            });
        });
    });
}
exports.openApiToMockFile = openApiToMockFile;
//# sourceMappingURL=OpenApiConverter.js.map